function [b,A,B,R,com_one,com_two,com_three] = kccaBound(p,a,b,kx,ky,k,d)

% This function will compute the bound on a kcca training 
% data and output
%
% Usage:
%   [b,A,B,R,com_one,com_two,com_three] = kccaBound(p,a,b,kx,ky,k,d);
%
% output b - the bound
%        A, B -
%        R -
%        com_one -
%        com_two -
%        com_three -
%
% input p - the correlation values
%       a, b - the features matrices
%       kx, ky - the kernel matrices
%       k - the number of feature vectors to use
%       d - the confidence value
%
% Written such that the largest eigenvalue (correlation) is the
% last component in the p vector. And similary with the relation to
% the matrices
%
% Written by David R. Hardoon 13/10/04
% Any modification or changes pls email to
% drh@ecs.soton.ac.uk
%
% Updated 01/02/05 - added some more comments and bug handling

if (nargin ~= 7)
  disp('incorrect number of inputs');
  help kccaBound;
  b=0;A=0;B=0;R=0;com_one=0;com_two=0;com_three=0;
  return;
end

n = size(a,2);
t = n-k+1;
k = t;

% compute the feature space
at = a(:,t:n);
bt = b(:,t:n);

A = sort(diag(at'*kx*at));
B = sort(diag(bt'*ky*bt));

s1 = size(A,1);
s2 = size(B,1);

if s1 > 1
  A = A(s1-1)*2;
else
  A = A(s1)*2;
end

if s2 > 1
  B = B(s2-1)*2;
else
  B = B(s2)*2;
end

AB = (A^2+B^2);

% compute the maximal distance
l = size(kx,1);
dis = zeros(l,1);

for i=1:l
	dis(i) = kx(i,i)+ky(i,i);
end

R = max(dis);

% compute the first component of the bound
com_one = 0;
for i=n:-1:k
	com_one = com_one + 2*(1-p(i));
end

% compute the second component

temp = 0;
for i=1:n
   temp = temp + (kx(i,i)+ky(i,i))^2;
end

com_two = ((4*AB*k)/l)*sqrt(temp);

% compute the third component

com_three = 3*R*AB*sqrt(log(2/d)/(2*l));

b = com_one + com_two + com_three;
